function g=edcinit(x,q,version);

% Initialisation of evidential distance-based classifier with arbitrary class labels
% (each training example may be labeled by a belief function)
%
% g=edcinit(x,q,version);
%
% Input :
% x = matrix training vectors, size (n,d), where n=number of examples, d=input dimension
% q = class labels expressed as commonalities, size (n,2^c), where c=number of classes
%     The order of belief masses is the same as assume in Philippe Smets FMT toolbox
% version = 1 (scalar parameter is returned) or 2 (vector parameter of
%           size (n,1) is returned)
% Output :
% g = optimized  parameter
% err = value of the error criterion 
%
% Uses the FMT package developed by Philippe Smets 
%
% References
%
% The basic method is described in
%
% T. Denoeux. A k-nearest neighbor classification rule based on Dempster-Shafer theory. 
%    IEEE Transactions on Systems, Man and Cybernetics, 25(05):804-813, 1995.
%
% The application of the method with possibilistic labels (a special case of the training
% data considered in this program),as well as error criterion 1, are discussed in
%
% T. Denoeux and L. M. Zouhal. Handling possibilistic labels in pattern classification 
%     using evidential reasoning. Fuzzy Sets and Systems, 122(3):47-62, 2001.
%
% Criteria 1 and 2 are presented in
%
% P. Vannoorenbergue and T. Denoeux. Handling uncertain labels in multiclass problems using 
%     belief decision trees IPMU'2002, Annecy, France, July 2002.


[n,d]=size(x);
C=size(q,2);
c=round(log(C)./log(2));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% calcul des K ppv dans l'ens. d'app.
%keyboard
D=zeros(n,n);
P=zeros(n,c);
for i = 1:n,
   D(:,i)=sum(((ones(n,1)*x(i,:))-x).^2,2);
   P(i,:)=betp1(qtom(q(i,:)));
end;
gik=D*P./((ones(size(D))-eye(size(D)))*P);
g=1./sum(gik.*P,2);

if version==1,
   g=mean(g);
end;
