function [err,g,iopt]=optimK(x,q,K,g0,a,crit);

% Optimisation of number of neighbours in evidential distance-based classifier
% [err,g,iopt]=optimK(x,q,K,a)
% Input :
% x = training set, size (n,d)
% q = class labels expressed as commonalities, size (n,2^c)
% K = vector of integers (nb of neighbors to be tested)
% g0 = initial value of scale parameter
% a = scalar parameter (default=0.99)
% crit = optimization criterion. For each test pattern,
%     crit=1 -> err=1-sum_{omega in Omega} phat(omega)*q({omega})
%               where phat= output pignistic probability
%               q = learning commonality function (label)
%     crit=2 -> err=1-sum_{omega in Omega} phat(omega)*p(omega) 
%               where phat= output pignistic probability
%               p = pignistic probability derived from learning commonality function
% Output :
% err = vector of error criterion values
% g = vector of optimized scalar parameter
% iopt = index of minimum error. The optimimum K-value is K(iopt)
%
% Uses:
% - the FMT package developed by Philippe Smets 
% - edcfit
%
% References
%
% The basic method is described in
%
% T. Denoeux. A k-nearest neighbor classification rule based on Dempster-Shafer theory. 
%    IEEE Transactions on Systems, Man and Cybernetics, 25(05):804-813, 1995.
%
% The application of the method with possibilistic labels (a special case of the training
% data considered in this program),as well as error criterion 1, are discussed in
%
% T. Denoeux and L. M. Zouhal. Handling possibilistic labels in pattern classification 
%     using evidential reasoning. Fuzzy Sets and Systems, 122(3):47-62, 2001.
%
% Criteria 1 and 2 are presented in
%
% P. Vannoorenbergue and T. Denoeux. Handling uncertain labels in multiclass problems using 
%     belief decision trees IPMU'2002, Annecy, France, July 2002.



if nargin < 5,
   a=0.99;
end;

ik=length(K);

g=zeros(ik,1);
err=g;
for i=1:ik,
   [g(i),err(i)]=edcfit(x,q,K(i),0,g0,a,crit);
end;
[errmin iopt]=min(err);
